"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createColorFunction = exports.createCoverFunc = exports.createInjectMixinTemplateFunction = void 0;
const cover_func_template_1 = __importDefault(require("../templates/cover-func-template"));
const upperFirst_1 = __importDefault(require("lodash/upperFirst"));
const when_replace_1 = require("../utils/when-replace");
const context_1 = require("../context");
const util = __importStar(require("../../client/util"));
/**
 * 创建注入函数
 * @param funcName 函数名
 */
const createInjectMixinTemplateFunction = (funcName) => {
    const templateName = `cover${(0, upperFirst_1.default)(funcName)}Template`;
    return (result) => {
        try {
            // postcss-less 无法解析 @functions 的用法
            // 所以直接通过修改通配生成后的字符文件内容进行注入
            result.css = cover_func_template_1.default[templateName] + result.css;
        }
        catch (error) {
            console.error('injection template error: ', error);
        }
        return result;
    };
};
exports.createInjectMixinTemplateFunction = createInjectMixinTemplateFunction;
/**
 * 创建覆盖函数
 * @param funcName 函数名
 * @returns
 */
const createCoverFunc = (funcName) => {
    return (atRule) => {
        return (0, when_replace_1.whenColorFuncReplace)(atRule, (str) => !!str && str.indexOf(`${funcName}(`) !== -1, (str) => {
            if (funcName === 'color') {
                return (str
                    .replace(/\s/g, '')
                    .replace(/color\(~?`?([^`]+)+(?=\)$)*/, (_, $1) => $1)
                    // hack
                    .replace('`)', ''));
            }
            const coverName = `cover${(0, upperFirst_1.default)(funcName)}`;
            const isFunctionVariables = str.includes(';');
            const replaceRE = str.includes('~`')
                ? new RegExp(`(${funcName})(\\(([.*\\s\\S][?=^;])+?=\\))`, 'g')
                : new RegExp(`(${funcName})(\\([.*\\s\\S]+\\))`);
            if (!replaceRE.test(str)) {
                return str
                    .replace(new RegExp(`${funcName}\\(`, 'g'), coverName + '(')
                    .replace(/%/g, '');
            }
            return str.replace(replaceRE, (_, $1, $2) => {
                let value = $2.replace(/\s/g, '');
                if (value.startsWith('(') && value.endsWith(')')) {
                    value = value.substring(1, value.length - 1);
                }
                let varName = value;
                let varNumber = '';
                if (value.includes(',')) {
                    const [p1, p2] = value.split(',');
                    varName = p1;
                    varNumber = p2;
                }
                value = value.substring(1, value.length - 1);
                if (varName.includes('@')) {
                    const [s, v] = varName.split('@');
                    varName = s.includes("'") ? `${s}@${v}` : `'@{${v}}'`;
                }
                else {
                    const intNumber = parseInt(varNumber);
                    if (util.staticColorList.includes(funcName)) {
                        return util[funcName](varName, intNumber);
                    }
                }
                if (varNumber.includes('%')) {
                    varNumber = varNumber.replace(/%$/, '');
                }
                if (varNumber.includes('@')) {
                    const match = varNumber.match(/@([\w-]+)/);
                    const name = match?.[1];
                    if (name) {
                        varNumber = `'@{${name}}'`;
                    }
                }
                const newFuncName = $1.replace(new RegExp(funcName, 'g'), coverName);
                handleCoverVariables(isFunctionVariables, funcName, varName, varNumber);
                let result = `${newFuncName}(${varName}${varNumber ? `,${varNumber}` : ''})`;
                result =
                    result.includes('~`') || str.includes('~`')
                        ? result
                        : `~\`${result}\``;
                if (!matchBrackets(result)) {
                    result = result.replace(/;?$/, ');');
                }
                return result.replace(/%/, '');
            });
        });
    };
};
exports.createCoverFunc = createCoverFunc;
function handleCoverVariables(isFunctionVariables, funcName, varName, varNumber) {
    if (!isFunctionVariables) {
        const match = varName.match(/\{(.*)\}/);
        if (match) {
            const name = match[1];
            // 小于1
            let num = parseInt(varNumber);
            if (num < 1) {
                num = num * 10;
            }
            const variablesName = `${name}--${funcName}--${num}`;
            context_1.antDesignColorMap.set(variablesName, util.__ANT_DESIGN_FUNCTION_TOKEN__);
        }
    }
}
/**
 * 创建相关函数
 * @param funcName
 * @returns
 */
function createColorFunction(funcName) {
    const injectFunc = (0, exports.createInjectMixinTemplateFunction)(funcName);
    const coverFunc = (0, exports.createCoverFunc)(funcName);
    return [coverFunc, injectFunc];
}
exports.createColorFunction = createColorFunction;
function matchBrackets(str) {
    var n = 0;
    for (let s of str) {
        if (s === '(') {
            n++;
        }
        if (s === ')') {
            if (n === 0) {
                return false;
            }
            n--;
        }
    }
    return n === 0;
}
